/*
 * Copyright (C) 2003 Jean-Philippe Hergott <hergott@free.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "errno.h"
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include "sys/wait.h"

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)

void
xcave_fill_filtered_model (gpointer		data,
							gint			colonne)
{
	GtkWidget			*TreeList;
	GtkWidget			*EntryColor;
	GtkTreeStore		*TreeList_filtered_model;
    GList 				*tmp_list_stock;
	GtkWidget			*treeview;
	GtkTreeModel 		*model;
	GtkTreeSelection	*selection;
	GtkTreeIter			iter;
	GtkTreeIter			child;
	GtkTreeIter			grandson;
	GtkTreeIter			parent;
	GtkTreeIter			grandpa;

    ptrVin 	    		tmpVin;
    const gchar			*pcolor;
	gchar				*pAppellation;
	gchar				*pRegion;
	gchar				*pcountry;
	gint				tmpcol;
	gint 				nbrow;

    XCAVE_DEBUG_B;

    tmpVin = g_malloc (sizeof (Vin));

	/*
	 * cleanup the treelist 
	 */
	TreeList = lookup_widget(window_xcave,"treeview_xcave_wine_information");
	TreeList_filtered_model	=(GtkTreeStore *)  gtk_tree_view_get_model((GtkTreeView *)TreeList); /* C */
	gtk_tree_store_clear(GTK_TREE_STORE(TreeList_filtered_model)); /* C */

	/*
	 * get the selected xcave_entry_color 
	 * needed to display only wine corresponding to the selected color
	 */
	EntryColor = lookup_widget(window_xcave,"xcave_entry_color");
	pcolor =(const gchar*) gtk_entry_get_text    ((GtkEntry *)EntryColor);

	/*
	 * get the selected row of the treeview 
	 * needed to display only wine belonging to country/region/appellation 
	 * selected
	 * if no row is selected then (tmpcol = -1) to display all wine on the 
	 * stock
	 */
	treeview = lookup_widget(window_xcave,"treeview_xcave_wine_origin");

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));  
	gtk_tree_selection_get_selected (selection, &model, &iter);

	pAppellation = g_malloc(appellation_size);

	nbrow = gtk_tree_selection_count_selected_rows(selection);
	
	if (nbrow == 0)
		tmpcol = -1;
	else {
		gtk_tree_model_get (model, &iter, COL_TREEVIEW, &pAppellation, -1);
		/*
		 * Determine level the selected data is 
		 */
		if (gtk_tree_model_iter_children(model, &child, &iter) == FALSE) {
			tmpcol = COL_TREELIST_APPELLATION;
		
			pRegion	= g_malloc(region_size);
			pcountry	= g_malloc(country_size);
		
			gtk_tree_model_iter_parent(model, &parent, &iter);
			gtk_tree_model_get (model, &parent, COL_TREEVIEW, &pRegion, -1);
			gtk_tree_model_iter_parent(model, &grandpa, &parent);
			gtk_tree_model_get (model, &grandpa, COL_TREEVIEW, &pcountry, -1);
		
		}
		else 
			if (gtk_tree_model_iter_children(model, &grandson, &child) == FALSE) 
				tmpcol = COL_TREELIST_REGION;
			else 
				tmpcol = COL_TREELIST_COUNTRY;
	}
		
	/*
	 * ok now it's time to play the game. I mean to glance through the stocks
	 * to display wine depending of different selection
	 */
	for(tmp_list_stock = list_stock; tmp_list_stock != NULL; tmp_list_stock = tmp_list_stock->next) {
        GtkTreeIter pIter;

		memset(tmpVin,'\0',sizeof(Vin));
		memcpy(tmpVin,tmp_list_stock->data,sizeof(Vin));

		if ( /* xcave_entry_color */
 			 ( 
			   (strcasecmp(pcolor,_("ALL")) == 0)
			   ||
			   (strcasecmp(pcolor,_("All")) == 0)
			   ||
		       (strcasecmp(pcolor,tmpVin->color) == 0)
			  )
			&&	/* treelist selection when xcave_entry_color is changed */	
			 ( (tmpcol == COL_TREELIST_APPELLATION && strcasecmp(pAppellation,tmpVin->Appellation) == 0)
			   ||
			   (tmpcol == COL_TREELIST_REGION && strcasecmp(pAppellation,tmpVin->Region) == 0)
			   ||		   (tmpcol == COL_TREELIST_COUNTRY && strcasecmp(pAppellation,tmpVin->country) == 0)
			   ||
			   (tmpcol == -1)
			 )
			&&	/* treelist selection when treelist is changed */	
			 ( (colonne == COL_TREELIST_APPELLATION && 
				strcasecmp(data,tmpVin->Appellation) == 0 &&
				strcasecmp(pRegion,tmpVin->Region)   == 0 &&
				strcasecmp(pcountry,tmpVin->country)       == 0
			    )
			   ||
			   (colonne == COL_TREELIST_REGION && strcasecmp(data,tmpVin->Region) == 0)
			   ||
			   (colonne == COL_TREELIST_COUNTRY && strcasecmp(data,tmpVin->country) == 0)
			   ||
			   (colonne == COL_TREELIST_COLOR && strcasecmp(data,tmpVin->color) == 0)
			   ||
			   (colonne == COL_TREELIST_COLOR && strcasecmp(data,_("All")) == 0)
			   ||
			   (colonne == COL_TREELIST_COLOR && strcasecmp(data,_("ALL")) == 0)
			 )
		   && /* quantity to display drinking wine that are not deleted from
			   * the stock
			   */
		     ( (QT == 0 && tmpVin->qty != 0 )  || ( QT == 1 ) )
			) {
				tmpVin->Maturite = xcave_compute_maturity_value(tmpVin->qty,tmpVin->vintage,tmpVin->Conserv);
				gtk_tree_store_append(TreeList_filtered_model, &pIter, NULL);/* C */
				gtk_tree_store_set(TreeList_filtered_model, &pIter, COL_TREELIST_MILLESIME, (guint)tmpVin->vintage,
									COL_TREELIST_QTY, (guint)tmpVin->qty,
									COL_TREELIST_DOMAIN, tmpVin->Nom,
									COL_TREELIST_COUNTRY, tmpVin->country,
									COL_TREELIST_REGION, tmpVin->Region,
									COL_TREELIST_APPELLATION, tmpVin->Appellation,
									COL_TREELIST_COLOR, tmpVin->color,
									COL_TREELIST_MATURITE, tmpVin->Maturite,
									COL_TREELIST_PDATA, tmp_list_stock->data,
									COL_TREELIST_PGLIST, tmp_list_stock,
									-1);			
			}
	
	}

	gtk_tree_sortable_set_sort_column_id (	GTK_TREE_SORTABLE (TreeList_filtered_model),
											COL_TREELIST_MATURITE, 
											GTK_SORT_DESCENDING);

	gtk_tree_view_set_model(GTK_TREE_VIEW(TreeList), GTK_TREE_MODEL(TreeList_filtered_model));

	{
		gint nb_bottle = calculate_nb_bottle (CALCULATE_COLOR,(gchar *)pcolor,0);
        if ( nb_bottle != 0) {			
			sprintf (xcave_statusbar_txt, _("Number of bottles in cellar: %d (%s: %d)"),
			                        NbBottleStock,
                                    pcolor,
					                nb_bottle);
            xcave_statusbar_ctxtid = gtk_statusbar_push ((GtkStatusbar *)xcave_statusbar_v, xcave_statusbar_ctxtid, xcave_statusbar_txt);
		}  else {
	        sprintf (xcave_statusbar_txt, _("Number of bottles in cellar: %d"),NbBottleStock);
            xcave_statusbar_ctxtid = gtk_statusbar_push ((GtkStatusbar *)xcave_statusbar_v, xcave_statusbar_ctxtid, xcave_statusbar_txt);

			/* 2.4
			 sprintf (xcave_statusbar_txt, _("Number of bottles in cellar: %d dont %d %s"),
			                        NbBottleStock,
                                    calculate_nb_bottle_in_rack(1),
			                        pcolor);
            xcave_statusbar_ctxtid = gtk_statusbar_push ((GtkStatusbar *)xcave_statusbar_v, xcave_statusbar_ctxtid, xcave_statusbar_txt);
			*/

		} 
	}
	
	g_free(tmpVin);
	g_free(pAppellation);

    XCAVE_DEBUG_E;
}

void
/* 2.3.1. xcave_fill_model (GtkTreeStore * TreeView, GtkListStore * TreeList) */
xcave_fill_model (GtkTreeStore * storeview_origin, GtkTreeStore * storeview_information)
{
    GList 			*tmp_list_stock;
    ptrVin     		tmpVin;
	gchar			*pCountry;
	gchar			*pRegCountry;
	gchar			*pAppCountry;
	gchar			*pRegion;
	gchar			*pAppRegion;
	gchar			*pAppellation;

    XCAVE_DEBUG_B;

    tmpVin = g_malloc (sizeof (Vin));

	pCountry		= g_malloc(country_size);
	pRegCountry		= g_malloc(country_size);
	pAppCountry		= g_malloc(country_size);
	pRegion			= g_malloc(region_size);
	pAppRegion			= g_malloc(region_size);
	pAppellation	= g_malloc(appellation_size);

	strcpy (pCountry, ".");
    strcpy (pRegion, ".");
    strcpy (pAppellation, ".");
	strcpy (pRegCountry,".");
	strcpy (pAppCountry,".");
	strcpy (pAppRegion, ".");

	for(tmp_list_stock = list_stock; tmp_list_stock != NULL; tmp_list_stock = tmp_list_stock->next) {
        GtkTreeIter pIter;
        GtkTreeIter pIter2;
        GtkTreeIter pIter3;
        GtkTreeIter pIter4;

		memset(tmpVin,'\0',sizeof(Vin));
		memcpy(tmpVin,tmp_list_stock->data,sizeof(Vin));

		/*
		 * Affichage des vins consommés - dont la quantité est nulle dans le stocks
	     * mais dont la trace a neanmoins ete conservee
		*/
		if ( (QT == 0 && tmpVin->qty != 0 )  || ( QT == 1 ) ){
			if (  strcasecmp (tmpVin->country,pCountry)        != 0 ) {
				/* new country */
				gtk_tree_store_append(storeview_origin, &pIter, NULL);/* C */
				strncpy (pCountry,tmpVin->country,country_size);
				gtk_tree_store_set(storeview_origin, &pIter, COL_TREEVIEW, pCountry, -1);			
			}
			if (  strcasecmp (tmpVin->Region,pRegion)           != 0 ||
				  (  strcasecmp (tmpVin->Region,pRegion)        == 0 &&
				     strcasecmp (tmpVin->country,pRegCountry)   != 0    )) {
				/* new region */
				gtk_tree_store_append(storeview_origin, &pIter2, &pIter);/* C */
				strncpy (pRegion, tmpVin->Region, region_size);
				gtk_tree_store_set(storeview_origin, &pIter2, COL_TREEVIEW, pRegion, -1);			
			}
		
			if (  (  strcasecmp (tmpVin->Appellation,pAppellation)     != 0)  
				  ||
				  (  strcasecmp (tmpVin->Appellation,pAppellation)     == 0 &&
				     strcasecmp (tmpVin->Region,pAppRegion)            != 0 ) 
			      ||
			      (  strcasecmp (tmpVin->Appellation,pAppellation)     == 0 &&
				     strcasecmp (tmpVin->Region,pAppRegion)            == 0 && 
					 strcasecmp (tmpVin->country,pAppCountry)             != 0)
			   ) {
				/* new appellation */ 
				gtk_tree_store_append(storeview_origin, &pIter3, &pIter2);/* C */
				strncpy (pAppellation, tmpVin->Appellation,  appellation_size);
				gtk_tree_store_set(storeview_origin, &pIter3, COL_TREEVIEW, pAppellation, -1);			
			}

			/* 2.3.1 gtk_list_store_append(TreeList, &pIter4); */ 
			gtk_tree_store_append(storeview_information, &pIter4, NULL);/* C */

			tmpVin->Maturite = xcave_compute_maturity_value(tmpVin->qty,tmpVin->vintage,tmpVin->Conserv);
			/* 2.3.1 gtk_list_store_set(TreeList, &pIter4,  */
			gtk_tree_store_set(storeview_information, &pIter4, 
						COL_TREELIST_MILLESIME, (guint)tmpVin->vintage,
						COL_TREELIST_QTY, (guint)tmpVin->qty,
						COL_TREELIST_DOMAIN, tmpVin->Nom,
						COL_TREELIST_COUNTRY, tmpVin->country,
						COL_TREELIST_REGION, tmpVin->Region,
						COL_TREELIST_APPELLATION, tmpVin->Appellation,
						COL_TREELIST_COLOR, tmpVin->color,
						COL_TREELIST_MATURITE, tmpVin->Maturite,
						COL_TREELIST_PDATA, tmp_list_stock->data,
						COL_TREELIST_PGLIST, tmp_list_stock,
						-1);			
			strncpy (pRegCountry,tmpVin->country,country_size);
			strncpy (pAppCountry,tmpVin->country,country_size);
			strncpy (pAppRegion, tmpVin->Region, region_size);
		}
	
	}

	gtk_tree_sortable_set_sort_column_id (	GTK_TREE_SORTABLE (storeview_information),
											COL_TREELIST_MATURITE, 
											GTK_SORT_DESCENDING);
	g_free(tmpVin);
    g_free(pCountry); 
    g_free(pRegCountry); 
    g_free(pAppCountry); 
    g_free(pAppRegion);
    g_free(pRegion);
    g_free(pAppellation);
    XCAVE_DEBUG_E;
}

void
on_xcave_realize (	GtkWidget * widget,
					gpointer user_data)
{
	GtkTreeStore		*storeview_xcave_wine_origin;
	// GtkListStore		*storeview_xcave_wine_information;
	GtkTreeStore		*storeview_xcave_wine_information;
	
	GtkWidget			*xcave_wine_origin_scrolledwindow;
	GtkWidget			*xcave_wine_information_scrolledwindow;
	GtkWidget			*treeview_xcave_wine_origin;
	GtkWidget			*treeview_xcave_wine_information;
	GtkWidget			*xcave_hpaned;
	GtkTreeViewColumn   *col;
	GtkCellRenderer     *renderer;

    XCAVE_DEBUG_B;

  window_xcave = widget;		/* assign main window in global for sub procedure treatments */

  xcave_statusbar_v = lookup_widget (widget, "xcave_statusbar");
  if (xcave_statusbar_v == NULL) g_warning ("Widget not found: xcave_statusbar");

  xcave_statusbar_ctxtid = gtk_statusbar_get_context_id ((GtkStatusbar *)xcave_statusbar_v, "default");
  xcave_statusbar_id = 0;

  /*
   * initialisation diverses et variees:
   *   (GList) liste du stock
   *   (GList) liste des country
   *   (GList) liste des regions
   *   (GList) liste des appellations
   *   (GList) liste des cepages
   */
	list_append_stock();
	
	list_append_rack();
	
	list_append_categ();
	list_append_country();
	list_append_region();
	list_append_appellation();
	list_append_cepage();
	
	list_country_append_stock();
	list_region_append_stock();
	list_appellation_append_stock();
	list_cepage_append_stock();

	list_country = g_list_sort(list_country,(GCompareFunc)cmp_country);
    list_region = g_list_sort(list_region,(GCompareFunc)cmp_region);
	list_appellation = g_list_sort(list_appellation,(GCompareFunc)cmp_appellation);
	list_cepage = g_list_sort(list_cepage,(GCompareFunc)cmp_cepage);

	/* stat */
	sprintf (xcave_statusbar_txt, _("Number of bottles in cellar : %d"), NbBottleStock);
    xcave_statusbar_ctxtid = gtk_statusbar_push ((GtkStatusbar *)xcave_statusbar_v, xcave_statusbar_ctxtid, xcave_statusbar_txt);

	/*
	* TreeView & TreeList init
	*/
	xcave_hpaned = lookup_widget(window_xcave,"xcave_hpaned");

	/*
	 * GtkListStore / GtkTreeStore - stockage de donnees
	 */

    /* Creates a new tree store as with n_columns columns each of the types passed in. */
    storeview_xcave_wine_origin = gtk_tree_store_new(NUM_COLS_TREEVIEW, G_TYPE_STRING);

	/* Creates a new tree store as with n_columns columns each of the types passed in. */
	storeview_xcave_wine_information = gtk_tree_store_new(NUM_COLS_TREELIST, 
				G_TYPE_UINT, 
				G_TYPE_UINT, 
				G_TYPE_STRING, 
				G_TYPE_STRING, 
				G_TYPE_STRING, 
				G_TYPE_STRING, 
				G_TYPE_STRING, 
				G_TYPE_UINT,
				G_TYPE_POINTER,
				G_TYPE_POINTER);

	/*
	 * WINE ORIGIN TREE VIEW
	 */ 
													
	/* create scrollbar */
	xcave_wine_origin_scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (xcave_wine_origin_scrolledwindow);
	gtk_paned_pack1 (GTK_PANED (xcave_hpaned), xcave_wine_origin_scrolledwindow, TRUE, TRUE);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (xcave_wine_origin_scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	
	/*
	 * GtkTreeView / GtkTreeModel - presentation 
	 */

	/* create TreeView */
	treeview_xcave_wine_origin = gtk_tree_view_new_with_model (GTK_TREE_MODEL(storeview_xcave_wine_origin));
	gtk_widget_show (treeview_xcave_wine_origin);
	gtk_container_add (GTK_CONTAINER (xcave_wine_origin_scrolledwindow), treeview_xcave_wine_origin);
	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview_xcave_wine_origin), TRUE);
	/*
	 * 2.3.1 alternance de couleur de fond pour chaque ligne
	 */
	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeview_xcave_wine_origin),TRUE);
	
	/* --- TreeView Column #1 --- */
    renderer = gtk_cell_renderer_text_new();
    col = gtk_tree_view_column_new_with_attributes(_("Wine origin"),renderer,"text", COL_TREEVIEW,NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview_xcave_wine_origin), col);

	/*
	 * WINE INFORMATION TREE VIEW
	 */ 

	/* create scrollbar */
	xcave_wine_information_scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (xcave_wine_information_scrolledwindow);
	gtk_paned_pack2 (GTK_PANED (xcave_hpaned), xcave_wine_information_scrolledwindow, TRUE, TRUE);
	GTK_WIDGET_UNSET_FLAGS (xcave_wine_information_scrolledwindow, GTK_CAN_FOCUS);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (xcave_wine_information_scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	
	/* create TreeList */
	treeview_xcave_wine_information = gtk_tree_view_new_with_model (GTK_TREE_MODEL(storeview_xcave_wine_information));
	gtk_widget_show (treeview_xcave_wine_information);
	gtk_container_add (GTK_CONTAINER (xcave_wine_information_scrolledwindow), treeview_xcave_wine_information);
	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview_xcave_wine_information), TRUE);
	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeview_xcave_wine_information),TRUE);
	
	/* --- TreeList Column #1 --- */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, _("Vintage"));
	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview_xcave_wine_information), col);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_set_cell_data_func(col, renderer, maturite_cell_data_function, NULL, NULL);
	gtk_tree_view_column_add_attribute(col, renderer, "text", COL_TREELIST_MILLESIME);
	gtk_tree_view_column_set_sort_column_id(col,COL_TREELIST_MILLESIME);
	
	/* --- TreeList Column #2 --- */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, _("Qty"));
	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview_xcave_wine_information), col);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_set_cell_data_func(col, renderer, maturite_cell_data_function, NULL, NULL);
	gtk_tree_view_column_add_attribute(col, renderer, "text", COL_TREELIST_QTY);
	gtk_tree_view_column_set_sort_column_id(col,COL_TREELIST_QTY);
	
	/* --- TreeList Column #3 --- */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, _("Domain"));
	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview_xcave_wine_information), col);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_set_cell_data_func(col, renderer, maturite_cell_data_function, NULL, NULL);
	gtk_tree_view_column_add_attribute(col, renderer, "text", COL_TREELIST_DOMAIN);
	gtk_tree_view_column_set_sort_column_id(col,COL_TREELIST_DOMAIN);
	
	/* --- TreeList Column #4 -- COUNTRY --- */
    renderer = gtk_cell_renderer_text_new();
    col = gtk_tree_view_column_new_with_attributes("label",renderer,"text", COL_TREELIST_COUNTRY,NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview_xcave_wine_information), col);
	gtk_tree_view_column_set_visible (col,FALSE);
	
	/* --- TreeList Column #5 -- REGION --- */
    renderer = gtk_cell_renderer_text_new();
    col = gtk_tree_view_column_new_with_attributes("label",renderer,"text", COL_TREELIST_REGION,NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview_xcave_wine_information), col);
	gtk_tree_view_column_set_visible (col,FALSE);
	
	/* --- TreeList Column #6 -- APPELLATION --- */
    renderer = gtk_cell_renderer_text_new();
    col = gtk_tree_view_column_new_with_attributes("label",renderer,"text", COL_TREELIST_APPELLATION,NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview_xcave_wine_information), col);
	gtk_tree_view_column_set_visible (col,FALSE);
	
	/* --- TreeList Column #7 -- COLOR --- */
    renderer = gtk_cell_renderer_text_new();
    col = gtk_tree_view_column_new_with_attributes("label",renderer,"text", COL_TREELIST_COLOR,NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview_xcave_wine_information), col);
	gtk_tree_view_column_set_visible (col,FALSE);

	/* --- TreeList Column #8 -- MATURITE --- */
    renderer = gtk_cell_renderer_text_new();

    /*
	 * 2.2.6
	 * col = gtk_tree_view_column_new_with_attributes("label",renderer,"gint", COL_TREELIST_MATURITE,NULL);
	 * warning lors de l execution 
	 * GLib-GObject-WARNING **: IA__g_object_set_property: object class `GtkCellRendererText' has no property named `gint'
	 */
    col = gtk_tree_view_column_new_with_attributes("label",renderer,"text", COL_TREELIST_MATURITE,NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview_xcave_wine_information), col);
	gtk_tree_view_column_set_visible (col,FALSE);
	
	/* --- TreeList Column #9 -- GList data --- */
    renderer = gtk_cell_renderer_text_new();
	/*
	 * 2.2.6
	 *  GLib-GObject-WARNING **: unable to set property `text' of type `gchararray' from value of type `gpointer'
	 * col = gtk_tree_view_column_new_with_attributes("label",renderer,"text", COL_TREELIST_PDATA,NULL);
	 * gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList), col);
	 * gtk_tree_view_column_set_visible (col,FALSE);
	 */
	
	/* --- TreeList Column #10 -- GList --- */
    renderer = gtk_cell_renderer_text_new();
	/*
	 * 2.2.6
	 *  GLib-GObject-WARNING **: unable to set property `text' of type `gchararray' from value of type `gpointer'
	 * col = gtk_tree_view_column_new_with_attributes("label",renderer,"text", COL_TREELIST_PGLIST,NULL);
	 * gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList), col);
	 * gtk_tree_view_column_set_visible (col,FALSE);
	 */

	xcave_fill_model (storeview_xcave_wine_origin, storeview_xcave_wine_information);

	/* destroy model automatically with view */
	g_object_unref(storeview_xcave_wine_origin);
	g_object_unref(storeview_xcave_wine_information);
	
	/*
	 * signal for doube click management
	 */
	g_signal_connect ((gpointer) treeview_xcave_wine_information, "row_activated",
					G_CALLBACK (on_TreeList_double_clicked),
					NULL);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT (window_xcave, xcave_wine_origin_scrolledwindow, "xcave_wine_origin_scrolledwindow");
	GLADE_HOOKUP_OBJECT (window_xcave, treeview_xcave_wine_origin, "treeview_xcave_wine_origin");
	GLADE_HOOKUP_OBJECT (window_xcave, xcave_wine_information_scrolledwindow, "xcave_wine_information_scrolledwindow");
	GLADE_HOOKUP_OBJECT (window_xcave,treeview_xcave_wine_information, "treeview_xcave_wine_information");

	window_detail_origine = NO_BUTTON;
	
    XCAVE_DEBUG_E;
}

void
quick_message (const gchar * buf)
{

    /* desactive window_xcave */
    gtk_widget_set_sensitive(window_detail, FALSE);

    GtkWidget	*dialog;
    dialog = gtk_message_dialog_new ((GtkWindow *)window_detail,
                                     GTK_DIALOG_DESTROY_WITH_PARENT,
                                     GTK_MESSAGE_WARNING,
                                     GTK_BUTTONS_CLOSE,
				     "%s",
                                     buf);
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);

    gtk_widget_set_sensitive(window_detail, TRUE);
}

/*
 * row selected from TreeView -> filter 
 * TreeList display 
 */
void
on_TreeView_cursor_changed             (GtkTreeView     *treeview,
                                        gpointer         user_data)
{
	GtkTreeModel 		*model;
	GtkTreeSelection	*selection;
	GtkTreeIter			iter;
	GtkTreeIter			child;
	GtkTreeIter			grandson;
	gchar				*pAppellation;

    XCAVE_DEBUG_B;
	pAppellation			= g_malloc(appellation_size);

	g_return_if_fail (GTK_IS_TREE_VIEW (treeview));
	selection = gtk_tree_view_get_selection (treeview);  

	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
		return;

	gtk_tree_model_get (model, &iter, COL_TREEVIEW, &pAppellation, -1);

	/*
	 * Determine level the selected data is 
	 */
	if (gtk_tree_model_iter_children(model, &child, &iter) == FALSE) {
		xcave_fill_filtered_model(pAppellation,COL_TREELIST_APPELLATION);
	}
	else {
		if (gtk_tree_model_iter_children(model, &grandson, &child) == FALSE) {
			xcave_fill_filtered_model(pAppellation,COL_TREELIST_REGION);
		}
		else {
			xcave_fill_filtered_model(pAppellation,COL_TREELIST_COUNTRY);

		}
			
	}

	g_free(pAppellation);

    XCAVE_DEBUG_E;
}

/*
 * Double click on TreeList View -> display
 * GtkWindow "detail".
 */
void
on_TreeList_double_clicked              (GtkTreeView     *treeview,
                                        GtkTreePath     *path,
                                        GtkTreeViewColumn *column,
                                        gpointer         user_data)
{
    XCAVE_DEBUG_B;
    GtkButton       *button = NULL;
	on_xcave_button_modif_clicked(button,&user_data);

    XCAVE_DEBUG_E;
}

/*
 * GtkWindow 'xcave'
 * GtkEntry 'color' changed 
 */
void
on_entry_color_changed (GtkEditable * editable,
						gpointer 	user_data)
{
	gchar				*pentry;

    XCAVE_DEBUG_B;

	pentry =(gchar *) gtk_entry_get_text    ((GtkEntry *)editable);
	xcave_fill_filtered_model(pentry,COL_TREELIST_COLOR);

    XCAVE_DEBUG_E;
}


